@@ -1,5 +1,5 @@
 from concurrent import futures
-from multiprocessing import Manager
+import multiprocessing
 from itertools import chain, imap, izip
 from UserList import UserList
 from UserDict import UserDict
@@ -25,11 +25,11 @@ class _Reducer(object):
     
     def __init__(self, func, init=None):
         self.func = func
-        self.list = Manager().list([init,])
+        self.list = multiprocessing.Manager().list([init,])
         
     def __call__(self, item):
-        init = self.func(self.list[0], item)
-        self.list[0] = init
+        aggregate = self.func(self.list[0], item)
+        self.list[0] = aggregate
     
     @property
     def result(self):
@@ -90,7 +90,7 @@ class ParallelGen(ParallelSeq):
         return self.__class__(i for i in self._map(_filter, self, ) if i)
         
     def flatten(self):
-        '''if the list consists of several sequences, those will be chained in one'''
+        '''if the data source consists of several sequences, those will be chained in one'''
         return self.__class__(chain(*self))
         
     def map(self, func):
@@ -165,4 +165,36 @@ class ParallelString(UserString, ParallelSeq):
         return self.map(func)
         
     def flatten(self):
-        return self
\ No newline at end of file
+        return self
+        
+
+def parallel(data_source):
+    if data_source.__class__.__name__ == 'generator':
+        return ParallelGen(data_source)
+    elif data_source.__class__.__name__ == 'function':
+        if data_source().__class__.__name__ == 'generator':
+            return ParallelGen(data_source())
+    else:
+        try:
+            iterator = iter(data_source)
+        except TypeError:
+            raise TypeError('supplied data source must be a generator, a generator function or an iterable, not %s' % data_source.__class__.__name_)
+        if iterator.__class__.__name__ in ['listiterator', 'tupleiterator', 'setiterator']:
+            return ParallelList(data_source)
+        elif iterator.__class__.__name__ == 'dictionary-keyiterator':
+            return ParallelDict(data_source)
+        elif iterator.__class__.__name__ == 'iterator':
+            return ParallelString(data_source)
+    raise TypeError('supplied data source must be a generator, a generator function or an iterable, not %s' % data_source.__class__.__name__)
+    
+    
+def lazy_parallel(data_source):
+    if data_source.__class__.__name__ == 'function':
+        if data_source().__class__.__name__ == 'generator':
+            return ParallelGen(data_source())
+    else:
+        try:
+            iterator = iter(data_source)
+        except TypeError:
+            raise TypeError('supplied data source must be a generator, a generator function or an iterable, not %s' % data_source.__class__.__name__)     
+        return ParallelGen(data_source)
\ No newline at end of file
